@@ -13,7 +13,11 @@
 
 package main
 
-import "testing"
+import (
+	"testing"
+
+	"github.com/prometheus/prometheus/config"
+)
 
 func TestParse(t *testing.T) {
 	tests := []struct {
@@ -36,18 +40,6 @@ func TestParse(t *testing.T) {
 			input: []string{"-web.external-url", "'https://url/prometheus'"},
 			valid: false,
 		},
-		{
-			input: []string{"-storage.remote.influxdb-url", ""},
-			valid: true,
-		},
-		{
-			input: []string{"-storage.remote.influxdb-url", "http://localhost:8086/"},
-			valid: true,
-		},
-		{
-			input: []string{"-storage.remote.influxdb-url", "'https://some-url/'"},
-			valid: false,
-		},
 		{
 			input: []string{"-alertmanager.url", ""},
 			valid: true,
@@ -69,7 +61,6 @@ func TestParse(t *testing.T) {
 	for i, test := range tests {
 		// reset "immutable" config
 		cfg.prometheusURL = ""
-		cfg.influxdbURL = ""
 		cfg.alertmanagerURLs = stringset{}
 
 		err := parse(test.input)
@@ -85,7 +76,7 @@ func TestParseAlertmanagerURLToConfig(t *testing.T) {
 	tests := []struct {
 		url      string
 		username string
-		password string
+		password config.Secret
 	}{
 		{
 			url:      "http://alertmanager.company.com",
