@@ -15,6 +15,7 @@ package kubernetes
 
 import (
 	"io/ioutil"
+	"sync"
 	"time"
 
 	"github.com/prometheus/client_golang/prometheus"
@@ -23,12 +24,12 @@ import (
 	"github.com/prometheus/common/log"
 	"github.com/prometheus/common/model"
 	"golang.org/x/net/context"
-	"k8s.io/client-go/1.5/kubernetes"
-	"k8s.io/client-go/1.5/pkg/api"
-	apiv1 "k8s.io/client-go/1.5/pkg/api/v1"
-	"k8s.io/client-go/1.5/pkg/util/runtime"
-	"k8s.io/client-go/1.5/rest"
-	"k8s.io/client-go/1.5/tools/cache"
+	"k8s.io/apimachinery/pkg/util/runtime"
+	"k8s.io/client-go/kubernetes"
+	"k8s.io/client-go/pkg/api"
+	apiv1 "k8s.io/client-go/pkg/api/v1"
+	"k8s.io/client-go/rest"
+	"k8s.io/client-go/tools/cache"
 )
 
 const (
@@ -59,12 +60,13 @@ func init() {
 	}
 }
 
-// Kubernetes implements the TargetProvider interface for discovering
+// Discovery implements the TargetProvider interface for discovering
 // targets from Kubernetes.
-type Kubernetes struct {
-	client kubernetes.Interface
-	role   config.KubernetesRole
-	logger log.Logger
+type Discovery struct {
+	client             kubernetes.Interface
+	role               config.KubernetesRole
+	logger             log.Logger
+	namespaceDiscovery *config.KubernetesNamespaceDiscovery
 }
 
 func init() {
@@ -75,8 +77,16 @@ func init() {
 	}
 }
 
+func (d *Discovery) getNamespaces() []string {
+	namespaces := d.namespaceDiscovery.Names
+	if len(namespaces) == 0 {
+		namespaces = []string{api.NamespaceAll}
+	}
+	return namespaces
+}
+
 // New creates a new Kubernetes discovery for the given role.
-func New(l log.Logger, conf *config.KubernetesSDConfig) (*Kubernetes, error) {
+func New(l log.Logger, conf *config.KubernetesSDConfig) (*Discovery, error) {
 	var (
 		kcfg *rest.Config
 		err  error
@@ -111,10 +121,10 @@ func New(l log.Logger, conf *config.KubernetesSDConfig) (*Kubernetes, error) {
 				CAFile:   conf.TLSConfig.CAFile,
 				CertFile: conf.TLSConfig.CertFile,
 				KeyFile:  conf.TLSConfig.KeyFile,
+				Insecure: conf.TLSConfig.InsecureSkipVerify,
 			},
-			Insecure: conf.TLSConfig.InsecureSkipVerify,
 		}
-		token := conf.BearerToken
+		token := string(conf.BearerToken)
 		if conf.BearerTokenFile != "" {
 			bf, err := ioutil.ReadFile(conf.BearerTokenFile)
 			if err != nil {
@@ -126,7 +136,7 @@ func New(l log.Logger, conf *config.KubernetesSDConfig) (*Kubernetes, error) {
 
 		if conf.BasicAuth != nil {
 			kcfg.Username = conf.BasicAuth.Username
-			kcfg.Password = conf.BasicAuth.Password
+			kcfg.Password = string(conf.BasicAuth.Password)
 		}
 	}
 
@@ -136,75 +146,100 @@ func New(l log.Logger, conf *config.KubernetesSDConfig) (*Kubernetes, error) {
 	if err != nil {
 		return nil, err
 	}
-	return &Kubernetes{
-		client: c,
-		logger: l,
-		role:   conf.Role,
+	return &Discovery{
+		client:             c,
+		logger:             l,
+		role:               conf.Role,
+		namespaceDiscovery: &conf.NamespaceDiscovery,
 	}, nil
 }
 
 const resyncPeriod = 10 * time.Minute
 
 // Run implements the TargetProvider interface.
-func (k *Kubernetes) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
-	rclient := k.client.Core().GetRESTClient()
+func (d *Discovery) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
+	rclient := d.client.Core().RESTClient()
 
-	switch k.role {
+	namespaces := d.getNamespaces()
+
+	switch d.role {
 	case "endpoints":
-		elw := cache.NewListWatchFromClient(rclient, "endpoints", api.NamespaceAll, nil)
-		slw := cache.NewListWatchFromClient(rclient, "services", api.NamespaceAll, nil)
-		plw := cache.NewListWatchFromClient(rclient, "pods", api.NamespaceAll, nil)
-		eps := NewEndpoints(
-			k.logger.With("kubernetes_sd", "endpoint"),
-			cache.NewSharedInformer(slw, &apiv1.Service{}, resyncPeriod),
-			cache.NewSharedInformer(elw, &apiv1.Endpoints{}, resyncPeriod),
-			cache.NewSharedInformer(plw, &apiv1.Pod{}, resyncPeriod),
-		)
-		go eps.endpointsInf.Run(ctx.Done())
-		go eps.serviceInf.Run(ctx.Done())
-		go eps.podInf.Run(ctx.Done())
+		var wg sync.WaitGroup
 
-		for !eps.serviceInf.HasSynced() {
-			time.Sleep(100 * time.Millisecond)
-		}
-		for !eps.endpointsInf.HasSynced() {
-			time.Sleep(100 * time.Millisecond)
-		}
-		for !eps.podInf.HasSynced() {
-			time.Sleep(100 * time.Millisecond)
-		}
-		eps.Run(ctx, ch)
+		for _, namespace := range namespaces {
+			elw := cache.NewListWatchFromClient(rclient, "endpoints", namespace, nil)
+			slw := cache.NewListWatchFromClient(rclient, "services", namespace, nil)
+			plw := cache.NewListWatchFromClient(rclient, "pods", namespace, nil)
+			eps := NewEndpoints(
+				d.logger.With("kubernetes_sd", "endpoint"),
+				cache.NewSharedInformer(slw, &apiv1.Service{}, resyncPeriod),
+				cache.NewSharedInformer(elw, &apiv1.Endpoints{}, resyncPeriod),
+				cache.NewSharedInformer(plw, &apiv1.Pod{}, resyncPeriod),
+			)
+			go eps.endpointsInf.Run(ctx.Done())
+			go eps.serviceInf.Run(ctx.Done())
+			go eps.podInf.Run(ctx.Done())
 
+			for !eps.serviceInf.HasSynced() {
+				time.Sleep(100 * time.Millisecond)
+			}
+			for !eps.endpointsInf.HasSynced() {
+				time.Sleep(100 * time.Millisecond)
+			}
+			for !eps.podInf.HasSynced() {
+				time.Sleep(100 * time.Millisecond)
+			}
+			wg.Add(1)
+			go func() {
+				defer wg.Done()
+				eps.Run(ctx, ch)
+			}()
+		}
+		wg.Wait()
 	case "pod":
-		plw := cache.NewListWatchFromClient(rclient, "pods", api.NamespaceAll, nil)
-		pod := NewPod(
-			k.logger.With("kubernetes_sd", "pod"),
-			cache.NewSharedInformer(plw, &apiv1.Pod{}, resyncPeriod),
-		)
-		go pod.informer.Run(ctx.Done())
+		var wg sync.WaitGroup
+		for _, namespace := range namespaces {
+			plw := cache.NewListWatchFromClient(rclient, "pods", namespace, nil)
+			pod := NewPod(
+				d.logger.With("kubernetes_sd", "pod"),
+				cache.NewSharedInformer(plw, &apiv1.Pod{}, resyncPeriod),
+			)
+			go pod.informer.Run(ctx.Done())
 
-		for !pod.informer.HasSynced() {
-			time.Sleep(100 * time.Millisecond)
+			for !pod.informer.HasSynced() {
+				time.Sleep(100 * time.Millisecond)
+			}
+			wg.Add(1)
+			go func() {
+				defer wg.Done()
+				pod.Run(ctx, ch)
+			}()
 		}
-		pod.Run(ctx, ch)
-
+		wg.Wait()
 	case "service":
-		slw := cache.NewListWatchFromClient(rclient, "services", api.NamespaceAll, nil)
-		svc := NewService(
-			k.logger.With("kubernetes_sd", "service"),
-			cache.NewSharedInformer(slw, &apiv1.Service{}, resyncPeriod),
-		)
-		go svc.informer.Run(ctx.Done())
+		var wg sync.WaitGroup
+		for _, namespace := range namespaces {
+			slw := cache.NewListWatchFromClient(rclient, "services", namespace, nil)
+			svc := NewService(
+				d.logger.With("kubernetes_sd", "service"),
+				cache.NewSharedInformer(slw, &apiv1.Service{}, resyncPeriod),
+			)
+			go svc.informer.Run(ctx.Done())
 
-		for !svc.informer.HasSynced() {
-			time.Sleep(100 * time.Millisecond)
+			for !svc.informer.HasSynced() {
+				time.Sleep(100 * time.Millisecond)
+			}
+			wg.Add(1)
+			go func() {
+				defer wg.Done()
+				svc.Run(ctx, ch)
+			}()
 		}
-		svc.Run(ctx, ch)
-
+		wg.Wait()
 	case "node":
 		nlw := cache.NewListWatchFromClient(rclient, "nodes", api.NamespaceAll, nil)
 		node := NewNode(
-			k.logger.With("kubernetes_sd", "node"),
+			d.logger.With("kubernetes_sd", "node"),
 			cache.NewSharedInformer(nlw, &apiv1.Node{}, resyncPeriod),
 		)
 		go node.informer.Run(ctx.Done())
@@ -215,7 +250,7 @@ func (k *Kubernetes) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
 		node.Run(ctx, ch)
 
 	default:
-		k.logger.Errorf("unknown Kubernetes discovery kind %q", k.role)
+		d.logger.Errorf("unknown Kubernetes discovery kind %q", d.role)
 	}
 
 	<-ctx.Done()
