@@ -46,7 +46,10 @@ Prometheus.Graph.prototype.initialize = function() {
 
   // Draw graph controls and container from Handlebars template.
 
-  var options = {'pathPrefix': PATH_PREFIX};
+  var options = {
+    'pathPrefix': PATH_PREFIX,
+    'buildVersion': BUILD_VERSION
+  };
   jQuery.extend(options, self.options);
   self.graphHTML = $(Mustache.render(graphTemplate, options));
   self.el.append(self.graphHTML);
@@ -212,13 +215,12 @@ Prometheus.Graph.prototype.populateInsertableMetrics = function() {
           items: "all",
           matcher: function(item) {
             // If we have result for current query, skip
-            if (!self.fuzzyResult.query || self.fuzzyResult.query !== this.query) {
+            if (self.fuzzyResult.query !== this.query) {
               self.fuzzyResult.query = this.query;
               self.fuzzyResult.map = {};
-              self.fuzzyResult.result = fuzzy.filter(this.query.replace('_', ' '), metrics, {
+              self.fuzzyResult.result = fuzzy.filter(this.query.replace(/ /g, ''), metrics, {
                 pre: '<strong>',
-                post: '</strong>',
-                extract: function(el) { return el.replace('_', ' ') }
+                post: '</strong>'
               });
               self.fuzzyResult.result.forEach(function(r) {
                 self.fuzzyResult.map[r.original] = r;
@@ -237,7 +239,7 @@ Prometheus.Graph.prototype.populateInsertableMetrics = function() {
           },
 
           highlighter: function (item) {
-            return $('<div>' + self.fuzzyResult.map[item].string.replace(' ', '_') + '</div>')
+            return $('<div>' + self.fuzzyResult.map[item].string + '</div>')
           },
         });
         // This needs to happen after attaching the typeahead plugin, as it
@@ -384,6 +386,7 @@ Prometheus.Graph.prototype.submitQuery = function() {
     url = PATH_PREFIX + "/api/v1/query";
     success = function(json, textStatus) { self.handleConsoleResponse(json, textStatus); };
   }
+  self.params = params;
 
   self.queryXhr = $.ajax({
       method: self.queryForm.attr("method"),
@@ -413,7 +416,15 @@ Prometheus.Graph.prototype.submitQuery = function() {
           return;
         }
         var duration = new Date().getTime() - startTime;
-        self.evalStats.html("Load time: " + duration + "ms <br /> Resolution: " + resolution + "s");
+        var totalTimeSeries = 0;
+        if (xhr.responseJSON.data !== undefined) {
+          if (xhr.responseJSON.data.resultType === "scalar") {
+            totalTimeSeries = 1;
+          } else {
+            totalTimeSeries = xhr.responseJSON.data.result.length;
+          }
+        }
+        self.evalStats.html("Load time: " + duration + "ms <br /> Resolution: " + resolution + "s <br />" + "Total time series: " + totalTimeSeries);
         self.spinner.hide();
       }
   });
@@ -508,7 +519,21 @@ Prometheus.Graph.prototype.transformData = function(json) {
       color: palette.color()
     };
   });
-  Rickshaw.Series.zeroFill(data);
+  data.forEach(function(s) {
+    // Insert nulls for all missing steps.
+    var newSeries = [];
+    var pos = 0;
+    for (var t = self.params.start; t <= self.params.end; t += self.params.step) {
+      // Allow for floating point inaccuracy.
+      if (s.data.length > pos && s.data[pos].x < t + self.params.step / 100) {
+        newSeries.push(s.data[pos]);
+        pos++;
+      } else {
+        newSeries.push({x: t, y: null});
+      }
+    }
+    s.data = newSeries;
+  });
   return data;
 };
 
@@ -553,6 +578,43 @@ Prometheus.Graph.prototype.updateGraph = function() {
     min: "auto",
   });
 
+  // Find and set graph's max/min
+  if (self.isStacked() === true) {
+    // When stacked is toggled
+    var max = 0;
+    self.data.forEach(function(timeSeries) {
+      var currSeriesMax = 0;
+      timeSeries.data.forEach(function(dataPoint) {
+        if (dataPoint.y > currSeriesMax && dataPoint.y != null) {
+          currSeriesMax = dataPoint.y;
+        }
+      });
+      max += currSeriesMax;
+    });
+    self.rickshawGraph.max = max*1.05;
+    self.rickshawGraph.min = 0;
+  } else {
+    var min = Infinity;
+    var max = -Infinity;
+    self.data.forEach(function(timeSeries) {
+      timeSeries.data.forEach(function(dataPoint) {
+        if (dataPoint.y < min && dataPoint.y != null) {
+          min = dataPoint.y;
+        }
+        if (dataPoint.y > max && dataPoint.y != null) {
+          max = dataPoint.y;
+        }
+      });
+    });
+    if (min === max) {
+      self.rickshawGraph.max = max + 1;
+      self.rickshawGraph.min = min - 1;
+    } else {
+      self.rickshawGraph.max = max + (0.1*(Math.abs(max - min)));
+      self.rickshawGraph.min = min - (0.1*(Math.abs(max - min)));
+    }
+  }
+
   var xAxis = new Rickshaw.Graph.Axis.Time({ graph: self.rickshawGraph });
 
   var yAxis = new Rickshaw.Graph.Axis.Y({
@@ -820,7 +882,7 @@ function init() {
   });
 
   $.ajax({
-    url: PATH_PREFIX + "/static/js/graph_template.handlebar",
+    url: PATH_PREFIX + "/static/js/graph_template.handlebar?v=" + BUILD_VERSION,
     success: function(data) {
 
       graphTemplate = data;
