@@ -332,6 +332,9 @@ PromConsole.Graph = function(params) {
 
   this.params = params;
   this.rendered_data = null;
+  // Keep a reference so that further updates (e.g. annotations) can be made
+  // by the user in their templates.
+  this.rickshawGraph = null;
   PromConsole._graph_registry.push(this);
 
   /*
@@ -444,13 +447,30 @@ PromConsole.Graph.prototype._render = function(data) {
 
   // Get the data into the right format.
   var seriesLen = 0;
+
   for (var e = 0; e < data.length; e++) {
     for (var i = 0; i < data[e].data.result.length; i++) {
-      series[seriesLen++] = {
+      series[seriesLen] = {
             data: data[e].data.result[i].values.map(function(s) { return {x: s[0], y: self._parseValue(s[1])}; }),
             color: palette.color(),
             name: self._escapeHTML(nameFuncs[e](data[e].data.result[i].metric)),
       };
+			// Insert nulls for all missing steps.
+			var newSeries = [];
+			var pos = 0;
+			var start = self.params.endTime - self.params.duration;
+      var step = self.params.duration / this.graphTd.offsetWidth;
+			for (var t = start; t <= self.params.endTime; t += step) {
+				// Allow for floating point inaccuracy.
+				if (series[seriesLen].data.length > pos && series[seriesLen].data[pos].x < t + step / 100) {
+					newSeries.push(series[seriesLen].data[pos]);
+					pos++;
+				} else {
+					newSeries.push({x: t, y: null});
+				}
+			}
+			series[seriesLen].data = newSeries;
+      seriesLen++;
     }
   }
   this._clearGraph();
@@ -487,6 +507,9 @@ PromConsole.Graph.prototype._render = function(data) {
         }
       },
       yFormatter: function(y) {
+        if (y === null) {
+          return "";
+        }
         return this.params.yHoverFormatter(y) + this.params.yUnits;
       }.bind(this)
   });
@@ -504,6 +527,8 @@ PromConsole.Graph.prototype._render = function(data) {
   xAxis.render();
   yAxis.render();
   graph.render();
+
+  this.rickshawGraph = graph;
 };
 
 PromConsole.Graph.prototype._clearGraph = function() {
@@ -513,10 +538,19 @@ PromConsole.Graph.prototype._clearGraph = function() {
   while (this.legendDiv.lastChild) {
     this.legendDiv.removeChild(this.legendDiv.lastChild);
   }
+  this.rickshawGraph = null;
 };
 
 PromConsole.Graph.prototype._xhrs = [];
 
+PromConsole.Graph.prototype.buildQueryUrl = function(expr) {
+  var p = this.params;
+  return PATH_PREFIX + "/api/v1/query_range?query=" +
+    encodeURIComponent(expr) +
+    "&step=" + p.duration / this.graphTd.offsetWidth +
+    "&start=" + (p.endTime - p.duration) + "&end=" + p.endTime;
+};
+
 PromConsole.Graph.prototype.dispatch = function() {
   for (var j = 0; j < this._xhrs.length; j++) {
     this._xhrs[j].abort();
@@ -526,9 +560,7 @@ PromConsole.Graph.prototype.dispatch = function() {
   var pending_requests = this.params.expr.length;
   for (var i = 0; i < this.params.expr.length; ++i) {
     var endTime = this.params.endTime;
-    var url = PATH_PREFIX + "/api/v1/query_range?query=" + encodeURIComponent(this.params.expr[i]) +
-      "&step=" + this.params.duration / this.graphTd.offsetWidth + 
-      "&start=" + (endTime - this.params.duration) + "&end=" + endTime;
+    var url = this.buildQueryUrl(this.params.expr[i]);
     var xhr = new XMLHttpRequest();
     xhr.open('get', url, true);
     xhr.responseType = 'json';
